// ====================================================================
//  account.cc Lightning Cheque data structures & support code
//  Copyright (C) 1998 David Freese                                  
//                                                                   
//  This program is free software; you can redistribute it and/or    
//  modify it under the terms of the GNU General Public License as   
//  published by the Free Software Foundation; either version 2 of   
//  the License, or (at your option) any later version.              
//                                                                   
//  This program is distributed in the hope that it will be useful,  
//  but WITHOUT ANY WARRANTY; without even the implied warranty of   
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
//  GNU General Public License for more details.                     
//                                                                   
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software      
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        
//                                                                   
//    Author: Dave Freese                                            
//  Internet: dfreese@intrepid.net                                   
//                                                                   
// ====================================================================

#include "account.H"

//===================================================================
// Transaction implementation
//===================================================================
Transaction::Transaction()
{
  memset (szNumber, 0, MAXNUMB + 1);
  date.today();
  memset (szDescription, 0, MAXDESC + 1);
  memset (szMemo, 0, MAXMEMO + 1);
  mnyCredit = 0.0;
  mnyDebit = 0.0;
  bRec = false;
}

Transaction::Transaction( char *num, Date dte,
                          char *desc, char *memo, char *action,
                          Money credit, Money debit,
                          bool isrec )
{
  memset ( szNumber, 0, MAXNUMB + 1);
  strncpy( szNumber, num, MAXNUMB );

  date = dte;

  memset (szDescription, 0, MAXDESC + 1);
  strncpy( szDescription, desc, MAXDESC );

  memset (szMemo, 0, MAXMEMO + 1);
  strncpy( szMemo, memo, MAXMEMO );

  mnyCredit = credit;
  mnyDebit = debit;
  bRec = isrec; 
}

bool Transaction::operator<( const Transaction &x )
{
  if( this->date > x.date ) return false;
  if( this->date < x.date ) return true;
  if( strcmp( this->szNumber, x.szNumber ) < 0 )
    return true;
  return false;
}

bool Transaction::operator==( const Transaction &x )
{
  if( this->date != x.date ) return false;
  if( strcmp( this->szNumber, x.szNumber ) == 0) return true;
  return false;
}

bool Transaction::operator>( const Transaction &x )
{
  if( this->date < x.date ) return false;
  if( this->date > x.date ) return true;
  if( strcmp( this->szNumber, x.szNumber ) > 0 )
    return true;
  return false;
}

void Transaction::Tnum( char *num )
{
  memset (szNumber, 0, MAXNUMB + 1);
  strncpy( szNumber, num, MAXNUMB );
}

char * Transaction::Tnum()
{
  return szNumber;
}

void Transaction::Tdate( Date &dte )
{
  date = dte;
}

Date Transaction::Tdate()
{
  return date;
}

void Transaction::Tsetdate (int m, int d, int y)
{
  date.setDate (m, d, y);
}

void Transaction::Tdesc( char *desc )
{
  memset (szDescription, 0, MAXDESC + 1);
  strncpy( szDescription, desc, MAXDESC );
}

char * Transaction::Tdesc()
{
  return szDescription;
}

void Transaction::Tmemo( char *memo )
{
  memset (szMemo, 0, MAXMEMO + 1);
  strncpy( szMemo, memo, MAXMEMO );
}

char * Transaction::Tmemo()
{
  return szMemo;
}

void Transaction::Trec( bool isrec )
{
  bRec = isrec; 
}

bool Transaction::Trec()
{
  return bRec;
}

int Xcompare( const void *x1, const void *x2 )
{
  Transaction **X1 = (Transaction **)(x1), **X2 = (Transaction **)(x2);
  if( **X1 < **X2 ) return -1;
  if( **X1 > **X2 ) return 1;
  return 0;
}

//=================================================================
// Account implementation
//=================================================================

Account::Account()
{
  lastnbr = 0;
  *szName = 0;
  *szDesc = 0;
  *szNotes = 0;
  numTransactions = 0;
  ptrTransaction = 0;
  for( int i = 0; i < MAXX; i++ ) {
    xaction[i] = 0;
  }
  Abalance = 0.0;
}

Account::Account( char *name, char *desc, char *notes )
{
  *szName = 0;
  *szDesc = 0;
  *szNotes = 0;
  numTransactions = 0;
  ptrTransaction = 0;
  for( int i = 0; i < MAXX; i++ ) {
    xaction[i] = 0;
  }
  Abalance = 0.0;
  memset (szName, 0, MAXNAME + 1);
  memset (szDesc, 0, MAXDESC + 1);
  memset (szNotes, 0, MAXNOTE + 1);
  strncpy( szName, name, MAXNAME );
  strncpy( szDesc, desc, MAXDESC );
  strncpy( szNotes, notes, MAXNOTE );
}

char *Account::AccName()
{
  return szName;
}

void Account::AccName( const char *s )
{
  memset (szName, 0, MAXNAME + 1);
  strncpy( szName, s, MAXNAME );
}

char *Account::AccDesc()
{
  return szDesc;
}

void Account::AccDesc( const char *s )
{
  memset (szDesc, 0, MAXDESC + 1);
  strncpy( szDesc, s, MAXDESC );
}

char *Account::AccNotes()
{
  return szNotes;
}

Account::~Account()
{
  for( int i = 0; i < MAXX; i++ )
    if( xaction[i] ) {
      delete xaction[i];
    }
}

void Account::adjustBalance( int rec )
{
  Money diff;

  if (numTransactions > 0) {
    for( int i = rec; i < numTransactions; i++ ) {
      diff = xaction[rec]->mnyCredit - xaction[rec]->mnyDebit;
      if( i == 0 )
        xaction[rec]->RecBalance = diff;
      else
        xaction[rec]->RecBalance = xaction[i-1]->RecBalance + diff;
      rec++;
    }
    Abalance = xaction[numTransactions-1]->RecBalance;
  } else
    Abalance = 0.0;
}
  
void Account::LastNbr (char *s)
{
  unsigned int nbr;
  if (sscanf( s, "%u", &nbr) == 1)
    if (nbr > lastnbr )
      lastnbr = nbr;
}

void Account::add( Transaction &x )
{
  Transaction *ptrX = new Transaction;
  if( numTransactions == MAXX ) return;
  xaction[numTransactions++] = ptrX;
  *ptrX = x;
  if( numTransactions == 1 ) {
    Abalance = ptrX->mnyCredit - ptrX->mnyDebit;
    ptrX->RecBalance = Abalance;
  }
  else {
    Abalance += (ptrX->mnyCredit - ptrX->mnyDebit );
    ptrX->RecBalance = Abalance;
  }
  LastNbr (x.Tnum ());
}

void Account::del( int  rec )
{
  if( !xaction[rec] )
    return;
  
  unsigned int nbr;
  if (sscanf( xaction[rec]->Tnum(), "%u", &nbr) == 1) {
    if (nbr == lastnbr )
      lastnbr--;
    if (lastnbr < 0) lastnbr = 0;
  }

  delete xaction[rec];
  
  for( int i = rec; i < numTransactions; i++ ) {
    xaction[i] = xaction[i+1];
  }
  if (--numTransactions == 0) lastnbr = 0;
  adjustBalance( rec );
}

void Account::upd( int rec, Transaction &x )
{
  if( rec > -1 && rec < MAXX && xaction[rec] ) {
    Transaction *X = xaction[rec];
    strncpy( X->szNumber, x.szNumber, MAXNUMB );
    X->date = x.date;
    memset (X->szDescription, 0, MAXDESC + 1);
    memset (X->szMemo, 0, MAXMEMO + 1);
    strncpy( X->szDescription, x.szDescription, MAXDESC );
    strncpy( X->szMemo, x.szMemo, MAXMEMO );
    X->mnyCredit = x.mnyCredit; X->mnyDebit = x.mnyDebit;
    X->bRec = x.bRec; 
  }
  adjustBalance( rec );
  LastNbr (x.Tnum ());
}

void Account::sort()
{
  qsort( &xaction[0], 
         numTransactions,
         sizeof( Transaction * ),
         Xcompare );
  adjustBalance(0);
}

Transaction * Account::Xline( int n )
{
  if( xaction[n] )
    return xaction[n];
  return 0;
}

int Account::Xnbr()
{
  return numTransactions;
} 

Money Account::A_recbal( int rec )
{
  return xaction[rec]->RecBalance;
}

Money Account::A_balance()
{
  return Abalance;
}

Money Account::BankBalance()
{
  Money balance = 0.0;
  for( int i = 0; i < numTransactions; i++ )
    if( xaction[i]->bRec == true )
      balance += (xaction[i]->mnyCredit - xaction[i]->mnyDebit );
  return balance;
}

