// ==================================================================== 
// 
// Fl_Combo.H
// 
//  Copyright (C) 1999David Freese                                  
//                                                                    
//  This program is free software; you can redistribute it and/or     
//  modify itunder the terms of the GNU General Public License as    
//  published by the Free Software Foundation; either version 2 of    
//  the License, or (at your option) any later version.              
//                                                                    
//  This program is distributed in the hope that it will be useful,   
//  but WITHOUT ANY WARRANTY; without even the implied warranty of    
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     
//  GNU General Public License for more details.                     
//                                                                    
//  You should have received a copy of the GNU General Public License 
//  along with this program; if not, write to the Free Software       
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        
//                                                                    
//  Author: Dave Freese                                             
//  Internet: dfreese@intrepid.net                                   
//                                                                    
//====================================================================

// Popup menu with last-picked item displayed in button
//
// Revised : 26 December 99
//           Added ButtonLabel() to allow any of the @ formatting
//           characters to be specified for the button label.

#ifndef Fl_COMBO_H
#define Fl_COMBO_H

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Hold_Browser.H>

#ifdef WIN32
#define STRCASECMP stricmp
#else
#define STRCASECMP strcasecmp
#endif

#define INITCBOSIZE 64

class Fl_Combo;

class cbo_browser : public Fl_Hold_Browser {
public:
  cbo_browser( int x, int y, int w, int h, const char *l=0)
  : Fl_Hold_Browser( x, y, w, h, l) {}
  ~cbo_browser() {}
  int handle(int e) { 
    return (Fl_Browser_::handle(e));
  }
};

class winBrowser : public Fl_Window {
public:
  winBrowser(Fl_Widget *wc, int X, int Y, int W, int H, const char *l=0);
  int handle(int);
  int Status;
  int status();
  void status(int);
  void show();
  char **plist;
  int  sizelist;
  cbo_browser *Bwsr;
  static void browser_cb(Fl_Widget*, void*);
#define CBO_OPEN 0
#define CBO_SELECTED 1
#define CBO_CLOSED 2
};

class Fl_Combo : public Fl_Group  {

protected:
  Fl_Button  Btn;
  Fl_Input   Input;
//  Fl_Hold_Browser *Bwsr;
  Fl_Widget *basew;
  winBrowser *wBwsr;

  char **cbolist;
  int size;
  int maxsize;
  int xpos;
  int ypos;
  int width;
  int height;
  int numlines;
  int state;
  int selected;
  void incr_array();
  
public:
  Fl_Combo (int x, int y, int w, int h, char * = 0);

  void ButtonLabel (const char *);
  const char *value ();
  void value (const char *);
  int  itemnbr () {return selected;}
  void add (const char *);
  void sort ();
  void clear_cbo ();
  void align (int);
  void labelcolor (int);
  void textfont (int);
  void textsize (int);
  int  vsize() { return numlines; }
  void vsize (int n) { numlines = n; }
  void btn_cb_i ();
  int  getsize () { return size; }
  char **getlist () { return cbolist; }
  
};

#endif
