// ====================================================================
//  csvio.cc  import/export to '|' separated value (CSV) file
//             for Lightning Cheque                                  
//  Copyright (C) 1998 David Freese                                  
//                                                                   
//  This program is free software; you can redistribute it and/or    
//  modify it under the terms of the GNU General Public License as   
//  published by the Free Software Foundation; either version 2 of   
//  the License, or (at your option) any later version.              
//                                                                   
//  This program is distributed in the hope that it will be useful,  
//  but WITHOUT ANY WARRANTY; without even the implied warranty of   
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
//  GNU General Public License for more details.                     
//                                                                   
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software      
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        
//                                                                   
//    Author: Dave Freese                                            
//  Internet: dfreese@intrepid.net                                   
//                                                                   
// ====================================================================

#include "cheque.H"
#include <stdlib.h>
#include <stdio.h>

static char szCSVfname[128];

int ImportCSV()
{
  Transaction X;
  FILE *fImport;
  int  mnth, day, yr, i;
  char cRec,
       *szTnum,
       *szTdesc,
       *szTmemo,
       *szDebit,
       *szCredit,
       *szYr, *szMn, *szDy;
  double debit,
         credit;
  char line[200];

  if( numAcc == MAXACCTS ) return 0;

  if( isDirty )
    writeData( szFLCfile );

  char *p = fl_file_chooser("Open CSV File", "*.csv", "" );
  if( p && *p ) {
    strcpy( szCSVfname, p );
  }
  else return 0;

  if( ( fImport = fopen( szCSVfname, "r" ) ) == 0 )
    return 0;

  if( numAcc > 0 )
    for( int i = 0; i < numAcc; i ++ )
      if( strcmp( acnt[i]->AccName(), szCSVfname ) == 0 )
        strcat( szCSVfname, ".2nd" );
 
  if (numAcc < MAXACCTS) {
    acnt[numAcc] = new Account( szCSVfname , "", "" );

    memset( line, 0, 200);
    fscanf (fImport, "%[^\n]%*c", line );
    
    while (!feof(fImport)) {

      memset( line, 0, 200);
      fscanf (fImport, "%[^\n]%*c", line );
      if (strlen (line) == 0) break;

      cRec = line[0];
      szTnum = &line[2];

      for (i = 3; line[i] !=0 && line[i] != '|'; i++);
      line[i] = 0; i++;
      szYr = &line[i];

      for (; line[i] !=0 && line[i] != '/'; i++);
      line[i] = 0; i++;
      szMn = &line[i];

      for (; line[i] != 0 && line[i] != '/'; i++);
      line[i] = 0; i++;
      szDy = &line[i];

      for (; line[i] != 0 && line[i] != '|'; i++);
      line[i] = 0; i++;
      szTdesc = &line[i];

      for (; line[i] != 0 && line[i] != '|'; i++);
      line[i] = 0; i++;
      szTmemo = &line[i];

      for (; line[i] != 0 && line[i] != '|'; i++);
      line[i] = 0; i++;
      szDebit = &line[i];

      for (; line[i] != 0 && line[i] != '|'; i++);
      line[i] = 0; i++;
      szCredit = &line[i];

      debit  = atof (szDebit);
      credit = atof (szCredit);
      
      mnth = atoi (szMn);
      day = atoi (szDy);
      yr = atoi (szYr);
      
      X.Tnum (szTnum);
      X.Tmemo (szTmemo);
      X.Tdesc (szTdesc);
      if (debit > 0.0) {
        X.mnyDebit = debit;
        X.mnyCredit = 0.0;
      } else {
        X.mnyDebit = 0.0;
        X.mnyCredit = credit;
      }
      if (cRec == 'X')
        X.Trec (true);      
      else
        X.Trec (false);      
      X.Tsetdate (mnth, day, yr);
      acnt[numAcc]->add( X );
      
    }

    fclose (fImport );
  }

  numAcc++;
  isDirty = true;

  return 1;
}

void ExportCSV()
{
  Account *pA = acnt[ acctNbr ];
  Transaction *pX;
  int numX = pA->Xnbr();
  FILE *fExport;
  int numAcnt;

  strcpy( szCSVfname, pA->AccName() );
  strcat( szCSVfname, ".csv" );
  if( ( fExport = fopen( szCSVfname, "w" ) ) == 0 )
    return;

  fprintf( fExport,"REC|NUMBER|DATE|DESC|MEMO|DEBIT|CREDIT\n" );
  numX = pA->Xnbr();
  for( int i = 0; i < numX; i++ ) {
    pX = pA->Xline(i);
    fprintf( fExport,
             "%c|% 4s|%4d/%02d/%02d|%s|%s|%s|%s\n",
             pX->Trec() == true ? 'X' : ' ',
             pX->Tnum(),
             pX->Tdate().Year(), pX->Tdate().Month(), pX->Tdate().Day(),
             pX->Tdesc(),
             pX->Tmemo(),
             szMoney(pX->mnyDebit, 8),
             szMoney(pX->mnyCredit, 8) );
  }
  fclose( fExport );
  return;
}


