// ==================================================================== 
// 
// flcio.cc Fast Light Took Kit Checkbook Program file i/o functions 
// 
//  Copyright (C) 1998, 1999 David Freese                                  
//                                                                    
//  This program is free software; you can redistribute it and/or     
//  modify itunder the terms of the GNU General Public License as    
//  published by the Free Software Foundation; either version 2 of    
//  the License, or (at your option) any later version.              
//                                                                    
//  This program is distributed in the hope that it will be useful,   
//  but WITHOUT ANY WARRANTY; without even the implied warranty of    
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     
//  GNU General Public License for more details.                     
//                                                                    
//  You should have received a copy of the GNU General Public License 
//  along with this program; if not, write to the Free Software       
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        
//                                                                    
//  Author: Dave Freese                                             
//  Internet: dfreese@intrepid.net                                   
//                                                                    
//====================================================================

/********************************************************************
 * cheque flcio.c -- read & write data from a '*.flc' datafile      *  
 ********************************************************************/

#include "flcio.H"
#include "money.H"

//====================================================================
// Prototypes to read from an flc data file
//====================================================================

int  openData (char *fname);
void closeData ();
int  openImport (char *fname);
void importGroupsV1 ();
void importGroupsV2 ();

void readGroupsV1 ();
void readGroupsV2 ();
void readAccountV1 (int);
void readAccountV2 (int);
void readTransactionV1 (Transaction &);
void readTransactionV2 (Transaction &);
char *readString ();
void readDate (Date &);

void writeAccount (int nbr);
void writeTransaction (Transaction *trans);
void writeString (char *str);
void writeDate (Date date);

FILE *dfile;
int version = 0;
int numAcc = 0;
Money balance = 0.0;

bool  AccountWasOpen = false;

Account *acnt[MAXACCTS];  // max of 10 accounts

static char *magic = "FLC Account";  

int readData( char *fname )
{
  switch (openData (fname)) {
    case -1 : return 0;
    case 0  :
    case 1  : readGroupsV1();
              break;
    case 2  : readGroupsV2();
              break;
    default : return 0;
  } 
  closeData();
  return 1;
}

void closeData()
{
  fclose( dfile );
}

//====================================================================
//  openData
//  reads in the data from file datafile
//====================================================================

int openData( char *datafile )
{
  char *s;

  dfile = fopen( datafile, "r" );
  if( dfile == 0 )
    return -1;

  s = readString();
  if ( strcmp( s, magic ) != 0 ) {
    fclose( dfile );
    return 0;
  }

  fread( &version, sizeof( int ), 1, dfile );
  fread( &numAcc, sizeof( int ), 1, dfile );
  return version;
}

/********************************************************************
 * readGroup - version < 2.x.x                                      * 
 *   reads in a group of accounts                                   *
 ********************************************************************/

void readGroupsV1()
{
  int  err = 0;

  if( AccountWasOpen ) {
    for( int i = 0; i < 10; i++ ) {
      if( acnt[i] != 0 ) delete acnt[i];
      acnt[i] = 0;
    }
  } else {
    for( int i = 0; i < 10; i++ )
      acnt[i] = 0;
    AccountWasOpen = true;
  }

  if( numAcc > (MAXACCTS) ) {
    fprintf(stderr,"Too many accounts!" );
    exit(1);
  }

  /* read in the accounts */
  for( int i = 0; i < numAcc; i++ ) {
    balance = 0.0;
    readAccountV1(i);
  }

}


/********************************************************************
 * readGroup - version = 2.x.x                                      * 
 *   reads in a group of accounts                                   *
 ********************************************************************/

void readGroupsV2()
{
  int  err = 0;

  if( AccountWasOpen ) {
    for( int i = 0; i < 10; i++ ) {
      if( acnt[i] != 0 ) delete acnt[i];
      acnt[i] = 0;
    }
  } else {
    for( int i = 0; i < 10; i++ )
      acnt[i] = 0;
    AccountWasOpen = true;
  }

  if( numAcc > (MAXACCTS) ) {
    fprintf(stderr,"Too many accounts!" );
    exit(1);
  }

  /* read in the accounts */
  for( int i = 0; i < numAcc; i++ ) {
    balance = 0.0;
    readAccountV2(i);
  }

}


int numImport;

int importData( char *fname )
{
  switch (openImport (fname)) {
    case -1 : return 0;
    case 0  :
    case 1  : importGroupsV1();
              break;
    case 2  : importGroupsV2();
              break;
    default : return 0;
  }
  fclose(dfile);
  return 1;
}

//====================================================================
//  openImport
//  reads in the data from file datafile
//====================================================================

int openImport( char *datafile )
{
  char *s;

  dfile = fopen( datafile, "r" );
  if( dfile == 0 )
    return -1;

  s = readString();
  if ( strcmp( s, magic ) != 0 ) {
    fclose( dfile );
    return 0;
  }

  fread( &version, sizeof( int ), 1, dfile );
  fread( &numImport, sizeof( int ), 1, dfile );

  return version;
}

/********************************************************************
 * importGroups - version < 2.x.x                                   * 
 *   reads in a group of accounts                                   *
 ********************************************************************/

void importGroupsV1()
{
  int  err = 0;
  int  i;

  if( (numAcc + numImport) > (MAXACCTS) ) {
    fprintf(stderr,"Too many accounts!" );
    exit(1);
  }
  /* read in the accounts */
  for( i = 0; i < numImport; i++ ) {
    balance = 0.0;
    readAccountV1(numAcc + i);
  }
  numAcc += numImport;
}

/********************************************************************
 * importGroups - version = 2.x.x                                   * 
 *   reads in a group of accounts                                   *
 ********************************************************************/

void importGroupsV2()
{
  int  err = 0;
  int  i;

  if( (numAcc + numImport) > (MAXACCTS) ) {
    fprintf(stderr,"Too many accounts!" );
    exit(1);
  }
  /* read in the accounts */
  for( i = 0; i < numImport; i++ ) {
    balance = 0.0;
    readAccountV2(numAcc + i);
  }
  numAcc += numImport;
}

/********************************************************************
 * readAccount - version < 2.x.x                                    * 
 *   reads in the data for an account from the datafile             *
 *                                                                  * 
 ********************************************************************/

void readAccountV1( int nbr )
{
  int numTrans, accID, numGrps, numread;
  char flags, acctype;
  char *s, szTmp[128],
       szName[41], szDesc[41], szNotes[81];
  Transaction X;

// account ID #
  fread( &accID, sizeof(int), 1, dfile );
  fread( &flags, sizeof(char), 1, dfile );
  fread( &acctype, sizeof(char), 1, dfile );
   
// account name  
  s = readString();
  sprintf(szName,"%.40s", s );

// account description
  s = readString();
  sprintf(szDesc,"%.40s", s );

// account notes
  s = readString();
  sprintf(szNotes,"%.80s", s );

  if( nbr > 0 )
    for( int i = 0; i < nbr; i ++ )
      if( strcmp( acnt[i]->AccName(), szName ) == 0 )
        strcat( szName, ".2nd" );
 
  acnt[nbr] = new Account( szName, szDesc, szNotes );

  numread = fread( &numTrans, sizeof(int ), 1, dfile );
  if( numread == 0 ) return;

  for( int i = 0; i < numTrans; i++ ) {
    readTransactionV1( X );
    acnt[nbr]->add( X );
  }
  
  numread = fread( &numGrps, sizeof(int), 1, dfile );
  if( numread == 0 ) return;

  if (numGrps) {
    readGroupsV1 ();
  }

}

/********************************************************************
 * readAccount - version = 2.x.x                                    * 
 *   reads in the data for an account from the datafile             *
 *                                                                  * 
 ********************************************************************/

void readAccountV2( int nbr )
{
  int numTrans, numGrps, numread;
  char *s, szTmp[128],
       szName[41], szDesc[41], szNotes[81];
  Transaction X;
   
// account name  
  s = readString();
  sprintf(szName,"%.40s", s );

// account description
  s = readString();
  sprintf(szDesc,"%.40s", s );

// account notes
  s = readString();
  sprintf(szNotes,"%.80s", s );

  if( nbr > 0 )
    for( int i = 0; i < nbr; i ++ )
      if( strcmp( acnt[i]->AccName(), szName ) == 0 )
        strcat( szName, ".2nd" );
 
  acnt[nbr] = new Account( szName, szDesc, szNotes );

  numread = fread( &numTrans, sizeof(int ), 1, dfile );
  if( numread == 0 ) return;

  for( int i = 0; i < numTrans; i++ ) {
    readTransactionV2( X );
    acnt[nbr]->add( X );
  }
}

/********************************************************************
 * readTransactionV1  - version < 2                                 * 
 *   reads in the data for a transaction from the datafile          *
 ********************************************************************/

void readTransactionV1( Transaction &X )
{
  int itmp, credit, debit;
  Date date;
  char *s, c;
  double damount, dshares, dzero = 0.0;
  char temp[128];

// transaction #
  s = readString();
  X.Tnum( s );

// transaction date
  readDate( date );
  X.Tdate( date );

// transaction description
  s = readString();
  X.Tdesc( s );

// transaction memo  
  s = readString();
  X.Tmemo( s );

// transaction action
  s = readString();

// transaction category
  fread( &itmp, sizeof(int), 1, dfile );
  
// transaction reconciled
  fread( &c, sizeof( char ), 1, dfile );
  X.Trec( c == 'y' ? true : false );

// transaction amount
  fread( &damount, sizeof(double), 1, dfile );

// transaction share price
  fread( &dshares, sizeof( double ), 1, dfile );

// credit account number
  fread( &credit, sizeof( int ), 1, dfile );

// debit account number
  fread( &debit, sizeof( int ), 1, dfile );
  
  if( credit < 0 ) {
    X.mnyDebit = damount;
    X.mnyCredit = 0.0;
    damount *= -1.0;
  }
  else {
    X.mnyCredit = damount;
    X.mnyDebit = 0.0;
  }
}

/********************************************************************
 * readTransaction - version 2.x.x                                  * 
 *   reads in the data for a transaction from the datafile          *
 ********************************************************************/

void readTransactionV2( Transaction &X )
{
  Date date;
  char *s, c;
  long lAmount;

// transaction #
  s = readString();
  X.Tnum( s );

// transaction date
  readDate( date );
  X.Tdate( date );

// transaction description
  s = readString();
  X.Tdesc( s );

// transaction memo  
  s = readString();
  X.Tmemo( s );

// transaction reconciled
  fread( &c, sizeof( char ), 1, dfile );
  X.Trec( c == 'y' ? true : false );

// transaction amount
  fread( &lAmount, sizeof(long), 1, dfile );

  if( lAmount < 0 ) {
    X.mnyDebit.value (-lAmount);
    X.mnyCredit = 0.0;
  }
  else {
    X.mnyCredit.value (lAmount);
    X.mnyDebit = 0.0;
  }
}

/********************************************************************\
 * readString                                                       * 
 *   reads in a string (char *) from the datafile                   *
 * Return: the string                                               * 
\********************************************************************/
char * readString()
{
  int  size;
  static char str[1024];
  
  memset( str, 0, 1024 );
  fread( &size, sizeof(int), 1, dfile );
  if( size > 1023 )
    exit(1);  
  fread( str, size, 1, dfile );
  
  return str;
}

/********************************************************************\
 * readDate                                                         * 
 *   reads in a Date struct from the datafile                       *
 *                                                                  * 
 * Return: the Date struct                                          * 
\********************************************************************/

void readDate( Date &dt )
{
  int d, m, y;
  
  fread( &y, sizeof(int), 1, dfile );
  fread( &m, sizeof(int), 1, dfile );
  fread( &d, sizeof(int), 1, dfile );
  dt.setDate( m, d, y );
}

//=========================================================
// Write data file using xacc format
//=========================================================

/********************************************************************\
 * writeData                                                        * 
 *   flattens the program data and saves it in a file               * 
 *                                                                  * 
 * Args:   datafile - the file to store the data in                 * 
 * Return: -1 on failure                                            * 
\********************************************************************/
void writeData( char *datafile )
{
  dfile = fopen( datafile, "w" );
  if( dfile == 0 ) {
    fprintf(stderr, "Cannot open file" );
    exit(2);
  }

  writeString( magic );

  version = 2;
  fwrite( &version, sizeof( int ), 1, dfile );

// number of accounts in this file
  fwrite( &numAcc, sizeof( int ), 1, dfile );
// write each account in turn
  for( int i = 0; i < numAcc; i++ )
    writeAccount( i );

  fclose( dfile );
}


/********************************************************************\
 * writeAccount                                                     * 
 *   saves the data for an account to the datafile                  *
\********************************************************************/

void writeAccount( int nbr )
{
  char s[81];
  Account *pA = acnt[nbr];
  Transaction *pX;
  int numX;
  
  writeString( pA->AccName() );
  writeString( pA->AccDesc() );
  writeString( pA->AccNotes() );

// number transactions in this account
  numX = pA->Xnbr();
  fwrite( &numX, sizeof(int), 1, dfile );

  for( int i = 0; i < numX; i++ ) {
    pX = pA->Xline(i);
    writeTransaction( pX );
  }

}

/********************************************************************\
 * writeTransaction                                                 * 
 *   saves the data for a transaction to the datafile               *
 *                                                                  * 
\********************************************************************/
void writeTransaction( Transaction *trans )
{
  long lAmount;
  Account *xfer_acc;
  int  category = 0;
  char reconciled;

  writeString (trans->Tnum());  
  writeDate   (trans->Tdate());  
  writeString (trans->Tdesc());  
  writeString (trans->Tmemo());  
  
  reconciled = (trans->Trec() == true) ? 'y' : 'n';  
  fwrite( &reconciled, sizeof(char), 1, dfile );
  
  if (trans->mnyCredit.value() > 0) 
    lAmount = trans->mnyCredit.value();
  else
    lAmount = -(trans->mnyDebit.value());
  fwrite( &lAmount, sizeof(long), 1, dfile );
}

/********************************************************************\
 * writeString                                                      * 
\********************************************************************/
void writeString( char *str )
{
  int size;
  int tmp;

  if( str == 0 ) str = "";   /* protect against null arguments */
  
  size = strlen( str ) + 1;
  tmp = size;
  fwrite( &tmp, sizeof(int), 1, dfile );
  
  fwrite( str, size, 1, dfile );
}

/********************************************************************\
 * writeDate                                                        * 
\********************************************************************/

void writeDate( Date date )
{
  int tmp;
  
  tmp = date.Year();
  fwrite( &tmp, sizeof(int), 1, dfile );
  
  tmp = date.Month();
  fwrite( &tmp, sizeof(int), 1, dfile );
  
  tmp = date.Day();
  fwrite( &tmp, sizeof(int), 1, dfile );
}

/*********************** END OF FILE *********************************/

