// ==================================================================== 
// 
//  Money.H 
// 
//  Copyright (C) 1999David Freese                                  
//                                                                    
//  This program is free software; you can redistribute it and/or     
//  modify itunder the terms of the GNU General Public License as    
//  published by the Free Software Foundation; either version 2 of    
//  the License, or (at your option) any later version.              
//                                                                    
//  This program is distributed in the hope that it will be useful,   
//  but WITHOUT ANY WARRANTY; without even the implied warranty of    
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     
//  GNU General Public License for more details.                     
//                                                                    
//  You should have received a copy of the GNU General Public License 
//  along with this program; if not, write to the Free Software       
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        
//                                                                    
//  Author: Dave Freese                                             
//  Internet: dfreese@intrepid.net                                   
//                                                                    
//====================================================================

// value of money stored as a long int (32 bits on x86 computers)
// largest + value of money =  $21,474,836.47:  (2^(n-1)-1)/100
// largest - value of money = -$21,474,836.48: -(2^(n-1))/100
//
// following operators are overload:
//
//   = << >> == != < > + - * / += -= *= /=
//
// the raw value stored may be retrieved or stored using the
// value functions

#ifndef MONEY_H
#define MONEY_H

#include <iostream.h>

extern long conv;
extern int  decades;
class Money {
  friend ostream &operator<< (ostream &, Money &);
  friend istream &operator>> (istream &, Money &);
  friend char* szMoney (const Money &m, int n);
protected:
  long amt;
public:
  Money ();
  Money (long);
  Money (double);
  Money (const char *s);
  ~Money() {}
  
  void value (long v) {amt = v;}
  long value () {return amt;}
  char *szMoney( const Money &m );
  
  Money operator+ (const Money &) const;
  Money operator+ (double) const;
  
  Money operator- (const Money &) const;
  Money operator- (double) const;
  
  Money operator* (const Money &) const;
  Money operator* (double) const;
  
  Money operator/ (const Money &) const;
  Money operator/ (double) const;
  
  bool operator== (const Money &) const;
  bool operator== (double) const;
  
  bool operator!= (const Money &) const;
  bool operator!= (double) const;
  
  bool operator< (const Money &) const;
  bool operator< (double) const;
  
  bool operator> (const Money &) const;
  bool operator> (double) const;
  
  const Money &operator= (const Money &);
  const Money &operator= (double);
  const Money &operator= (const char *);

  const Money &operator+= (const Money &);
  const Money &operator+= (double);
    
  const Money &operator-= (const Money &);
  const Money &operator-= (double);
    
  const Money &operator*= (const Money &);
  const Money &operator*= (double);
    
  const Money &operator/= (const Money &);
  const Money &operator/= (double);
    
};

#endif
