// ==================================================================== 
// 
//  Money.cxx
// 
//  Copyright (C) 1999David Freese                                  
//                                                                    
//  This program is free software; you can redistribute it and/or     
//  modify itunder the terms of the GNU General Public License as    
//  published by the Free Software Foundation; either version 2 of    
//  the License, or (at your option) any later version.              
//                                                                    
//  This program is distributed in the hope that it will be useful,   
//  but WITHOUT ANY WARRANTY; without even the implied warranty of    
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     
//  GNU General Public License for more details.                     
//                                                                    
//  You should have received a copy of the GNU General Public License 
//  along with this program; if not, write to the Free Software       
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        
//                                                                    
//  Author: Dave Freese                                             
//  Internet: dfreese@intrepid.net                                   
//                                                                    
//====================================================================
#include "money.H"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

// decades = 2 for precision to penny
// conv = 10^decades

int  decades = 2;
long conv = 100;

// convert type Money to a string using the format xxxxx.xx
// n digits total in the field 
char* szMoney( const Money &m, int n )
{
  static char szM[10];
  long dollars, cents;
  char fmt[15];
  if (n <= 0 || n > 10) n = 8;
  sprintf (fmt,"%s%0d%s", "%", n - 2, "d.%02d");
  dollars = m.amt / conv;
  cents = abs(m.amt - conv*dollars);
  sprintf (szM, fmt, dollars, cents);
  return szM;
}


static long HPround ( double d )
{
  if (d == 0.0) return 0;
  if (d > 0.0) 
    return (long)floor (d*conv + 0.5);
  return (long)floor (d*conv);
}


ostream &operator<<( ostream &output, Money &m )
{
//  double d = (double)m.amt / conv;
//  output.precision(decades);
  long f = cout.flags();
  output << ( m.amt / conv) << '.';
  cout.setf (ios::right);
  cout.width (decades);
  cout.fill ('0');
  output << abs(m.amt - conv*(m.amt / conv));
  cout.flags (f);
  return output;
}

istream &operator>>( istream &input, Money &m )
{
  double d;
  input >> d;
  m = Money (HPround(d));
  return input;
}

Money::Money () 
{
  amt = 0;
}

Money::Money (long n) 
{
  amt = n;
}

Money::Money (double d) 
{
  amt = HPround (d);
}

static long str2mny (const char *s)
{
  double d = 0.0;
  int n = sscanf( s, "%lf", &d );
  if( n == 1 )
    return HPround (d);
  else
    return 0;
}

Money::Money( const char *s )
{
  amt = str2mny (s);
}

Money Money::operator+ ( const Money &m ) const
{
  return Money (amt + m.amt);
}


Money Money::operator+ ( double d) const
{
  return Money (amt + HPround(d));
}
    
Money Money::operator- ( const Money &m ) const
{
  return Money (amt - m.amt);
}

Money Money::operator- ( double d) const
{
  return Money (amt - HPround(d));
}
  
static long product (long m1, long m2)
{
  long up = m2 / conv;
  long lwr = m2 - up*conv;
  long p1 = m1 * up;
  long p2 = (m1 * lwr)/conv;
  return (p1+p2);
}

Money Money::operator* ( const Money & m) const
{
  return Money (product (amt, m.amt));
}

Money Money::operator* ( double d) const
{
  return Money ( HPround(amt * d / conv));
}
  
Money Money::operator/ ( const Money &m ) const
{
  return Money ((amt * conv) / m.amt);
}

Money Money::operator/ (double d) const
{
  return Money(HPround(amt / (d*conv)));
}
 
bool Money::operator== ( const Money &m ) const
{
  return (amt == m.amt);
}

bool Money::operator== ( double d) const
{
  return (amt == HPround(d));
}
  
bool Money::operator!= ( const Money &m ) const
{
  return (amt != m.amt);
}

bool Money::operator!= (double d) const
{
  return (amt != HPround(d));
}

  
bool Money::operator< ( const Money &m ) const
{
  return (amt < m.amt);
}

bool Money::operator< (double d) const
{
  return (amt < HPround(d));
}

  
bool Money::operator> ( const Money &m ) const
{
  return (amt > m.amt);
}

bool Money::operator> (double d) const
{
  return (amt > HPround(d));
}

  
const Money& Money::operator= (const Money &m )
{
  amt = m.amt;
  return *this;
}

const Money& Money::operator= (double d)
{
  amt = HPround(d);
  return *this;
}

const Money& Money::operator= (const char *s)
{
  amt = str2mny (s);
  return *this;
}

const Money& Money::operator+= (const Money &m)
{
  amt += m.amt;
  return *this;
}

const Money& Money::operator+= (double d)
{
  amt += HPround(d);
  return *this;
}

const Money& Money::operator-= (const Money &m)
{
  amt -= m.amt;
  return *this;
}

const Money& Money::operator-= (double d)
{
  amt -= HPround(d);
  return *this;
}

const Money& Money::operator*= (const Money &m)
{
  amt = product (amt, m.amt);
  return *this;
}

const Money& Money::operator*= (double d)
{
  amt = HPround(amt * d / conv);
  return *this;
}

const Money& Money::operator/= (const Money &m)
{
  amt = (amt * conv) / m.amt;
  return *this;
}

const Money& Money::operator/= (double d)
{
  amt = HPround(amt / (d*conv));
  return *this;
}

