// ====================================================================
//
//  plot.cc  Fast Light Took Kit Checkbook Program
//           data plotting routines
//
//  Copyright (C) 1998 David Freese                                  
//                                                                   
//  This program is free software; you can redistribute it and/or    
//  modify it under the terms of the GNU General Public License as   
//  published by the Free Software Foundation; either version 2 of   
//  the License, or (at your option) any later version.              
//                                                                   
//  This program is distributed in the hope that it will be useful,  
//  but WITHOUT ANY WARRANTY; without even the implied warranty of   
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
//  GNU General Public License for more details.                     
//                                                                   
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software      
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        
//                                                                   
//    Author: Dave Freese                                            
//  Internet: dfreese@intrepid.net                                   
//                                                                   
// ====================================================================

// Class definition for MD/MD plot
// db vs freq

#include "plot.H"
#include "cheque.H"

double MDPlot::MnyMax()
{
  return mnyMax;
}

void MDPlot::MnyMax( double v )
{
  mnyMax = v;
}

double MDPlot::MnyMin()
{
  return mnyMin;
}

void MDPlot::MnyMin( double v )
{
  mnyMin = v;
}

double MDPlot::YrsMax()
{
  return yrsMax;
}

void MDPlot::YrsMax( double v )
{
  yrsMax = v;
}

double MDPlot::YrsMin()
{
  return yrsMin;
}

void MDPlot::YrsMin( double v )
{
  yrsMin = v;
}

void MDPlot::draw()
{
  Account *a = acnt[acctNbr];
  double recbal;

  yrsMin = a->Xline(0)->date.Year();
  yrsMax = a->Xline(a->Xnbr()-1)->date.Year() + 1;
  mnyMax = -1.0e6;
  mnyMin = 1.0e6;
  for( int i = 0; i < a->Xnbr(); i++ ) {
    recbal = (a->A_recbal(i)).value()/conv;
    mnyMax = ( recbal > mnyMax ) ? recbal : mnyMax;
    mnyMin = ( recbal < mnyMin ) ? recbal : mnyMin;
  }
// round to nearest 1000;
  int imny = (int) (mnyMax / 1000 + 1);
  mnyMax = imny * 1000.0;
  imny = (int) (mnyMin / 1000 );
  mnyMin = imny * 1000.0;

  fl_clip(x(),y(),w(),h());
  fl_color(FL_BLACK);
  fl_rectf(x(),y(),w(),h());
  fl_push_matrix();
  fl_color(FL_GRAY);
// set up translate & scale for 0,0 at lower left corner of area
  fl_translate(x(),y() + h());
  fl_scale( w()/(yrsMax-yrsMin), -h()/(mnyMax-mnyMin ) );

  for( double dx = yrsMin; dx < yrsMax; dx++ ) {
    fl_begin_line();
    fl_vertex( dx - yrsMin, 0 );
    fl_vertex( dx - yrsMin, (mnyMax - mnyMin) );
    fl_end_line();
  }
  for( double dy = mnyMin;
       dy <= mnyMax;
       dy += ( (mnyMax-mnyMin < 1000 ) ? 100 : 500 ) ) {
    fl_begin_line();
    fl_vertex( 0, dy - mnyMin );
    fl_vertex( yrsMax - yrsMin, dy - mnyMin );
    fl_end_line();
  }

  PlotMny();
  PlotSlidingAvg();
  fl_pop_matrix();
  fl_pop_clip();
}

void MDPlot::PlotMny()
{
  Account *a = acnt[acctNbr];
  double x1, x2, y1, y2;
  fl_color( FL_GREEN );
  x1 = a->Xline(0)->date.dJulian() - yrsMin;
  y1 = (a->A_recbal(0)).value()/conv - mnyMin;
  for( int i = 0; i < a->Xnbr(); i++ ) {
    x2 = a->Xline(i)->date.dJulian() - yrsMin;
    y2 = (a->A_recbal(i)).value()/conv - mnyMin;
    fl_begin_line();
    fl_vertex( x1, y1 );
    fl_vertex( x2, y2 );
    fl_end_line();
    x1 = x2;
    y1 = y2;
  }
}

#define AVGNBR 40

void MDPlot::PlotSlidingAvg()
{
  Account *a = acnt[acctNbr];
  double x1, x2, y1, y2;
  int iX = a->Xnbr();

  if( iX < AVGNBR ) return;

  fl_color( FL_YELLOW );
 
  x1 = a->Xline(AVGNBR - 1)->date.dJulian() - yrsMin;

  y1 = 0.0;

  for( int j = 0; j < AVGNBR; j++ )
    y1 += ((a->A_recbal(j)).value()/conv - mnyMin);
  y1 = y1/(1.0 * AVGNBR);
  
  for( int i = AVGNBR - 1; i < iX; i++ ) {
    x2 = a->Xline(i)->date.dJulian() - yrsMin;
    y2 = 0.0;
    for( int j = 0; j < AVGNBR; j++ )
      y2 += ((a->A_recbal(i + j - (AVGNBR - 1))).value()/conv - mnyMin);
    y2 = y2 / (1.0 * AVGNBR);
    fl_begin_line();
    fl_vertex( x1, y1 );
    fl_vertex( x2, y2 );
    fl_end_line();
    x1 = x2;
    y1 = y2;
  }
}

