// ====================================================================
//  qifio.c -- read from and writing to Quicken Export format files     
//             for Lightning Cheque                                  
//  Copyright (C) 1998 David Freese                                  
//                                                                   
//  This program is free software; you can redistribute it and/or    
//  modify it under the terms of the GNU General Public License as   
//  published by the Free Software Foundation; either version 2 of   
//  the License, or (at your option) any later version.              
//                                                                   
//  This program is distributed in the hope that it will be useful,  
//  but WITHOUT ANY WARRANTY; without even the implied warranty of   
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
//  GNU General Public License for more details.                     
//                                                                   
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software      
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        
//                                                                   
//    Author: Dave Freese                                            
//  Internet: dfreese@intrepid.net                                   
//                                                                   
// ====================================================================

#include "qifio.H"
#include "money.H"

bool readok;

#ifndef TEST

void QIF_ImportData( char *fName )
{
  FILE *hFile;
  char szName[128];
  Transaction X;

  if( numAcc == MAXACCTS ) return;

  if( isDirty )
    writeData( szFLCfile );

  if( ( hFile = fopen( fName, "r" ) ) == 0 ) return;

  strcpy( szName, fName );

  if( numAcc > 0 )
    for( int i = 0; i < numAcc; i ++ )
      if( strcmp( acnt[i]->AccName(), szName ) == 0 )
        strcat( szName, ".2nd" );
 
  acnt[numAcc] = new Account( szName, "", "" );
  
  while (!feof( hFile ) ) {
    ClearQIFTransaction( X );
    while( QIF_ParseLine( QIF_ReadLine( hFile ), X ) == false );
    if (readok) acnt[numAcc]->add( X );
  }
  numAcc++;
  fclose( hFile );
  isDirty = true;
}

void cb_mImportQIF( Fl_Menu_ *m, void *data )
{
  Transaction X;
  char *p;

  p = fl_file_chooser("Select QIF File", "*.qif", "" );
  if (p && *p ) {
    QIF_ImportData( p );
    UpdateAccountMenu();
    acctNbr = 0;
    loadBrowser(); // start with account 0
    SetTitle();
  }
}

void ExportQIF()
{
  Account *pA = acnt[ acctNbr ];
  Transaction *pX;
  int numX = pA->Xnbr();
  FILE *fExport;
  int numAcnt;
  char szQIFfname[128];

  strcpy( szQIFfname, pA->AccName() );
  strcat( szQIFfname, ".qif" );
  if( ( fExport = fopen( szQIFfname, "w" ) ) == 0 )
    return;

  numX = pA->Xnbr();
  for( int i = 0; i < numX; i++ ) {
    pX = pA->Xline(i);
    fprintf( fExport, "N%4s\n",
             pX->Tnum());
    fprintf( fExport, "D%4d/%02d/%02d\n",
             pX->Tdate().Month(), pX->Tdate().Day(), pX->Tdate().Year() );
    fprintf( fExport, "C%c\n",
             pX->Trec() == true ? 'X' : ' ');
    fprintf( fExport, "T%8.2f\n",
             pX->mnyDebit > 0.0 ?
               -1.0*(pX->mnyDebit).value()/(conv) : 
               (pX->mnyCredit).value()/conv );
    fprintf( fExport, "P%s\n",
             pX->Tdesc());
// export memo field as a category to quicken
    fprintf( fExport, "L%s\n",         
             pX->Tmemo());
    fprintf( fExport, "^\n");        
  }
  fclose( fExport );
  return;
 }

void cb_mExportQIF( Fl_Menu_ *m, void *data )
{
  ExportQIF();
}

#endif


// --------------------------------------------------------------------
// QIF_ReadLine 
//   reads in one line of ASCII, until cr-nl
//
// Args:   fd -- file descriptor
// Return: pointer to static char buff containing ASCII
// --------------------------------------------------------------------

#define QIF_BUFF_SIZE 81

char * QIF_ReadLine( FILE *hFile ) 
{
  static char linebuff [QIF_BUFF_SIZE+1];
  int n;
  int nread;
  
  memset( linebuff, 0, QIF_BUFF_SIZE+1 );

// read chars from file until newline or buffer overflow
  n = 0;
  while ( n < QIF_BUFF_SIZE ) {
    nread = fread( &linebuff[n], sizeof (char), 1, hFile );
    if( sizeof(char) != nread )
      return 0;
    if( linebuff[n] == '\n' ) {
      linebuff[n] = 0;
      n--;
      if( linebuff[n] == '\r' )
         linebuff[n] = 0;
      break;
    }
    n++;
  }

  /* if newline not found, bust out */
  if ( n == QIF_BUFF_SIZE )
    return 0;
  return linebuff;
}


// --------------------------------------------------------------------
// QIF_ParseDate
//   parses date of the form MM/DD/YY
//
// Args:   date -- pointer to Date structure
// Args:   str -- pointer to string rep of date
// Return: void
// --------------------------------------------------------------------

void QIF_ParseDate (Date &dat, char * s ) 
{
  int dd = 0, mm = 0, yy = 0, nconv;
  nconv = sscanf( s, "%d/%d/%d", &mm, &dd, &yy );
  if (nconv > 0 ) dat.Month( mm );
  if (nconv > 1 ) dat.Day( dd );
  if (nconv > 2 ) dat.Year( yy < 100 ? 1900 + yy : yy );
}

// --------------------------------------------------------------------
// QIF_ParseAmount
//   parses dollar ammount of the form MMM,TTT,DDD.CC
//
// Args:   str -- pointer to string rep of sum
// Return: int -- in pennies
// --------------------------------------------------------------------

double QIF_ParseAmount (char * s) 
{
  double val = 0;
  int sign = 1;
  double post = 0;
  if( *s == '-' ) {
    sign = -1;
    s++;
  }
  while( *s ) {
    if( *s == ',' );
    else if( *s == '.' )
      post = 10;
    else if( isdigit( *s ) ) {
      if( post == 0) val = val*10 + ( *s - '0' );
      else {
         val = val + ( *s -'0' )/post;
         post *= 10;
      }
    }
    s++;
  }
  return (val * sign );  
}

bool QIF_ParseLine( char * s, Transaction & Xact )
{
  double amount;
  bool done = false;

  readok = false;

  if( s == 0 || *s == 0 )
    return true;

  if( strncmp( "!Type", s, 5 ) == 0 ) 
    return true;

  switch( s[0] ) {
    case 'D' : // Date
      QIF_ParseDate( Xact.date, &s[1] );
      break;
    case 'C' : // Cleared status
      if( s[1] == 'X' ) Xact.Trec( true );
      break;
    case 'T' : // Amount
      amount = QIF_ParseAmount( &s[1] );
      if( amount < 0 ) Xact.mnyDebit = -amount;
      else             Xact.mnyCredit = amount;
      break;
    case 'N' : // Number (check or reference )
      Xact.Tnum( &s[1] );
      break;
    case 'P' : // Payee/description
      Xact.Tdesc( &s[1] );
      break;
    case 'M' : // Memo
      Xact.Tmemo( &s[1] );
      break;
    case 'L' : // Use as Memo
      Xact.Tmemo( &s[1] );
      break;
    case '^' : // End of entry
      readok = true;
      done = true;
    case 'A' : // Address (up to 5 lines, 6th line is optional message)
    case 'S' : // Category in split
    case 'E' : // Memo in split
    case '$' : // Dollar amount in split
    default :
      break;
  }
  return done;
}

void ClearQIFTransaction ( Transaction & X )
{
  X.Tnum ("");
  X.Tdesc ("");
  X.Tmemo (""); 
  X.mnyDebit = 0.0;
  X.mnyCredit = 0.0;
  X.Trec ( false );
}

void QIF_Print( Transaction &X )
{
  Money dbt = X.mnyDebit,
        cdt = X.mnyCredit;
  X.Tdate().setFormat(0);
  cout << X.Tnum() << '|'
       << X.Tdate() << '|'
       << X.Tdesc() << '|'
       << X.Tmemo() << '|'
       << dbt << '|'
       << cdt << endl;
}

#ifdef TEST

void QIF_ParseFile( char *fName )
{
  Transaction X;

  FILE *hFile = fopen( fName, "r" );
  if (hFile == 0 ) return;
  while (!feof( hFile ) ) {
    ClearQIFTransaction( X );
    while ( QIF_ParseLine( QIF_ReadLine( hFile ), X ) == false );
    if (readok) QIF_Print( X );
  }
  fclose( hFile );
}

int main( int argc, char **argv )
{
  if( argc < 2 ) exit(0);
  QIF_ParseFile( argv[1] );
}

#endif

// ========================== END OF FILE ======================= 
